@@ -1,4 +1,5 @@
 """A selection of helper functions for building in pyspark."""
+
 import functools
 import itertools
 import logging
@@ -20,6 +21,7 @@ from pyspark.sql import SparkSession, Window, WindowSpec
 from pyspark.sql import functions as F
 from pyspark.sql import types as T
 
+from rdsa_utils.cdp.io.input import extract_database_name
 from rdsa_utils.logging import log_spark_df_schema
 
 logger = logging.getLogger(__name__)
@@ -76,8 +78,8 @@ def melt(
     df: SparkDF,
     id_vars: Union[str, Sequence[str]],
     value_vars: Union[str, Sequence[str]],
-    var_name: str = 'variable',
-    value_name: str = 'value',
+    var_name: str = "variable",
+    value_name: str = "value",
 ) -> SparkDF:
     """Melt a spark dataframe in a pandas like fashion.
 
@@ -145,11 +147,11 @@ def melt(
     )
 
     # Add to the DataFrame and explode, which extends the dataframe
-    _tmp = df.withColumn('_vars_and_vals', F.explode(_vars_and_vals))
+    _tmp = df.withColumn("_vars_and_vals", F.explode(_vars_and_vals))
 
     # We only want to select certain columns
     cols = id_vars + [
-        F.col('_vars_and_vals')[x].alias(x) for x in [var_name, value_name]
+        F.col("_vars_and_vals")[x].alias(x) for x in [var_name, value_name]
     ]
 
     return _tmp.select(*cols)
@@ -179,12 +181,18 @@ def to_spark_col(_func=None, *, exclude: Sequence[str] = None) -> Callable:
             varnames = func.__code__.co_varnames
             if args:
                 args = [
-                    _convert_to_spark_col(arg) if varnames[i] not in exclude else arg # noqa: E501
+                    (
+                        _convert_to_spark_col(arg)
+                        if varnames[i] not in exclude
+                        else arg
+                    )  # noqa: E501
                     for i, arg in enumerate(args)
                 ]
             if kwargs:
                 kwargs = {
-                    k: _convert_to_spark_col(kwarg) if k not in exclude else kwarg # noqa: E501
+                    k: (
+                        _convert_to_spark_col(kwarg) if k not in exclude else kwarg
+                    )  # noqa: E501
                     for k, kwarg in kwargs.items()
                 }
             return func(*args, **kwargs)
@@ -233,8 +241,7 @@ def map_column_names(df: SparkDF, mapper: Mapping[str, str]) -> SparkDF:
     If the column name is not in the mapper the name doesn't change.
     """
     cols = [
-        F.col(col_name).alias(mapper.get(col_name, col_name))
-        for col_name in df.columns
+        F.col(col_name).alias(mapper.get(col_name, col_name)) for col_name in df.columns
     ]
     return df.select(*cols)
 
@@ -367,7 +374,7 @@ def rank_numeric(
 
 def calc_median_price(
     groups: Union[str, Sequence[str]],
-    price_col: str = 'price',
+    price_col: str = "price",
 ) -> SparkCol:
     """Calculate the median price per grouping level.
 
@@ -384,7 +391,7 @@ def calc_median_price(
         A single entry for each grouping level, and its median price.
     """
     # Note median in [1,2,3,4] would return as 2 using below.
-    median = f'percentile_approx({price_col}, 0.5)'
+    median = f"percentile_approx({price_col}, 0.5)"
 
     return F.expr(median).over(Window.partitionBy(groups))
 
@@ -440,16 +447,12 @@ def convert_cols_to_struct_col(
 
     if not struct_cols:
         df = df.withColumn(
-            f'no_{struct_col_name}',
+            f"no_{struct_col_name}",
             F.lit(no_struct_col_value).cast(no_struct_col_type),
         )
-        struct_cols = [f'no_{struct_col_name}']
+        struct_cols = [f"no_{struct_col_name}"]
 
-    return (
-        df
-        .withColumn(struct_col_name, F.struct(*struct_cols))
-        .drop(*struct_cols)
-    )
+    return df.withColumn(struct_col_name, F.struct(*struct_cols)).drop(*struct_cols)
 
 
 def select_first_obs_appearing_in_group(
@@ -491,12 +494,7 @@ def select_first_obs_appearing_in_group(
         group=group,
         ascending=ascending,
     )
-    return (
-        df
-        .withColumn('rank', rank_by_date)
-        .filter(F.col('rank') == 1)
-        .drop('rank')
-    )
+    return df.withColumn("rank", rank_by_date).filter(F.col("rank") == 1).drop("rank")
 
 
 @log_spark_df_schema
@@ -531,13 +529,11 @@ def convert_struc_col_to_columns(
         *[col for col in df.columns if col not in struct_cols],
         # All columns identified as being struct type, but expand the struct
         # to individual columns using .* notation.
-        *[f'{col}.*' for col in struct_cols],
+        *[f"{col}.*" for col in struct_cols],
     )
 
-    if (
-        convert_nested_structs and
-        any(isinstance(field.dataType, T.StructType)
-        for field in df.schema.fields)
+    if convert_nested_structs and any(
+        isinstance(field.dataType, T.StructType) for field in df.schema.fields
     ):
         df = convert_struc_col_to_columns(df=df)
 
@@ -577,24 +573,18 @@ def cut_lineage(df: SparkDF) -> SparkDF:
     >>> new_df.count()
     3
     """
-    logger.info('Converting SparkDF to Java RDD.')
-
     try:
+        logger.info("Converting SparkDF to Java RDD.")
+
         jrdd = df._jdf.toJavaRDD()
         jschema = df._jdf.schema()
         jrdd.cache()
-        sql_ctx = df.sql_ctx
-        try:
-            java_sql_ctx = sql_ctx._jsqlContext
-        except AttributeError:
-            java_sql_ctx = sql_ctx._ssql_ctx
-
-        logger.info('Creating new SparkDF from Java RDD.')
-        new_java_df = java_sql_ctx.createDataFrame(jrdd, jschema)
-        new_df = SparkDF(new_java_df, sql_ctx)
+        spark = df.sparkSession
+        new_java_df = spark._jsparkSession.createDataFrame(jrdd, jschema)
+        new_df = SparkDF(new_java_df, spark)
         return new_df
-    except Exception:
-        logger.error('An error occurred during the lineage cutting process.')
+    except Exception as e:
+        logger.error(f"An error occurred during the lineage cutting process: {e}")
         raise
 
 
@@ -629,22 +619,22 @@ def find_spark_dataframes(
     frames = {}
 
     for key, value in locals_dict.items():
-        if key in ['_', '__', '___']:
+        if key in ["_", "__", "___"]:
             continue
 
         if isinstance(value, SparkDF):
             frames[key] = value
-            logger.info(f'SparkDF found: {key}')
+            logger.info(f"SparkDF found: {key}")
         elif (
             isinstance(value, dict)
             and value
             and isinstance(next(iter(value.values())), SparkDF)
         ):
             frames[key] = value
-            logger.info(f'Dictionary of SparkDFs found: {key}')
+            logger.info(f"Dictionary of SparkDFs found: {key}")
         else:
             logger.debug(
-                f'Skipping non-SparkDF item: {key}, Type: {type(value)}',
+                f"Skipping non-SparkDF item: {key}, Type: {type(value)}",
             )
 
     return frames
@@ -652,7 +642,7 @@ def find_spark_dataframes(
 
 def create_spark_session(
     app_name: Optional[str] = None,
-    size: Optional[Literal['small', 'medium', 'large', 'extra-large']] = None,
+    size: Optional[Literal["small", "medium", "large", "extra-large"]] = None,
     extra_configs: Optional[Dict[str, str]] = None,
 ) -> SparkSession:
     """Create a PySpark Session based on the specified size.
@@ -728,62 +718,65 @@ def create_spark_session(
     try:
         if size:
             size = size.lower()
-            valid_sizes = ['small', 'medium', 'large', 'extra-large']
+            valid_sizes = ["small", "medium", "large", "extra-large"]
             if size not in valid_sizes:
-                msg = (
-                    f"Invalid '{size=}'. "
-                    f"If specified must be one of {valid_sizes}."
-                )
+                msg = f"Invalid '{size=}'. If specified must be one of {valid_sizes}."
                 raise ValueError(msg)
 
         logger.info(
-            f"Creating a '{size}' Spark session..."
-            if size
-            else 'Creating a basic Spark session...',
+            (
+                f"Creating a '{size}' Spark session..."
+                if size
+                else "Creating a basic Spark session..."
+            ),
         )
 
         if app_name:
-            builder = SparkSession.builder.appName(f'{app_name}')
+            builder = SparkSession.builder.appName(f"{app_name}")
         else:
             builder = SparkSession.builder
 
         # fmt: off
-        if size == 'small':
+        if size == "small":
             builder = (
-                builder.config('spark.executor.memory', '1g')
-                .config('spark.executor.cores', 1)
-                .config('spark.dynamicAllocation.maxExecutors', 3)
-                .config('spark.sql.shuffle.partitions', 12)
+                builder.config("spark.executor.memory", "1g")
+                .config("spark.executor.cores", 1)
+                .config("spark.dynamicAllocation.maxExecutors", 3)
+                .config("spark.sql.shuffle.partitions", 12)
             )
-        elif size == 'medium':
+        elif size == "medium":
             builder = (
-                builder.config('spark.executor.memory', '6g')
-                .config('spark.executor.cores', 3)
-                .config('spark.dynamicAllocation.maxExecutors', 3)
-                .config('spark.sql.shuffle.partitions', 18)
+                builder.config("spark.executor.memory", "6g")
+                .config("spark.executor.cores", 3)
+                .config("spark.dynamicAllocation.maxExecutors", 3)
+                .config("spark.sql.shuffle.partitions", 18)
             )
-        elif size == 'large':
+        elif size == "large":
             builder = (
-                builder.config('spark.executor.memory', '10g')
-                .config('spark.yarn.executor.memoryOverhead', '1g')
-                .config('spark.executor.cores', 5)
-                .config('spark.dynamicAllocation.maxExecutors', 5)
-                .config('spark.sql.shuffle.partitions', 200)
+                builder.config("spark.executor.memory", "10g")
+                .config("spark.yarn.executor.memoryOverhead", "1g")
+                .config("spark.executor.cores", 5)
+                .config("spark.dynamicAllocation.maxExecutors", 5)
+                .config("spark.sql.shuffle.partitions", 200)
             )
-        elif size == 'extra-large':
+        elif size == "extra-large":
             builder = (
-                builder.config('spark.executor.memory', '20g')
-                .config('spark.yarn.executor.memoryOverhead', '2g')
-                .config('spark.executor.cores', 5)
-                .config('spark.dynamicAllocation.maxExecutors', 12)
-                .config('spark.sql.shuffle.partitions', 240)
+                builder.config("spark.executor.memory", "20g")
+                .config("spark.yarn.executor.memoryOverhead", "2g")
+                .config("spark.executor.cores", 5)
+                .config("spark.dynamicAllocation.maxExecutors", 12)
+                .config("spark.sql.shuffle.partitions", 240)
             )
 
         # Common configurations for all sizes
         builder = (
-            builder.config('spark.dynamicAllocation.enabled', 'true')
-            .config('spark.shuffle.service.enabled', 'true')
-            .config('spark.ui.showConsoleProgress', 'false')
+            # Dynamic Allocation
+            builder.config("spark.dynamicAllocation.enabled", "true")
+             .config("spark.dynamicAllocation.shuffleTracking.enabled", "true")
+             # Adaptive Query Execution
+             .config("spark.sql.adaptive.enabled", "true")
+             # General
+             .config("spark.ui.showConsoleProgress", "false")
         ).enableHiveSupport()
         # fmt: on
 
@@ -792,8 +785,233 @@ def create_spark_session(
             for key, value in extra_configs.items():
                 builder = builder.config(key, value)
 
-        logger.info('Spark session created successfully!')
+        logger.info("Spark session created successfully!")
         return builder.getOrCreate()
     except Exception as e:
-        logger.error(f'An error occurred while creating the Spark session: {e}')
+        logger.error(f"An error occurred while creating the Spark session: {e}")
+        raise
+
+
+def load_csv(
+    spark: SparkSession,
+    filepath: str,
+    keep_columns: Optional[List[str]] = None,
+    rename_columns: Optional[Dict[str, str]] = None,
+    drop_columns: Optional[List[str]] = None,
+    **kwargs,
+) -> SparkDF:
+    """Load a CSV file into a PySpark DataFrame.
+
+    spark
+        Active SparkSession.
+    filepath
+        The full path and filename of the CSV file to load.
+    keep_columns
+        A list of column names to keep in the DataFrame, dropping all others.
+        Default value is None.
+    rename_columns
+        A dictionary to rename columns where keys are existing column
+        names and values are new column names.
+        Default value is None.
+    drop_columns
+        A list of column names to drop from the DataFrame.
+        Default value is None.
+    kwargs
+        Additional keyword arguments to pass to the `spark.read.csv` method.
+
+    Returns
+    -------
+    SparkDF
+        PySpark DataFrame containing the data from the CSV file.
+
+    Notes
+    -----
+    Transformation order:
+    1. Columns are kept according to `keep_columns`.
+    2. Columns are dropped according to `drop_columns`.
+    3. Columns are renamed according to `rename_columns`.
+
+    Raises
+    ------
+    Exception
+        If there is an error loading the file.
+    ValueError
+        If a column specified in rename_columns, drop_columns, or
+        keep_columns is not found in the DataFrame.
+
+    Examples
+    --------
+    Load a CSV file with multiline and rename columns:
+
+    >>> df = load_csv(
+            spark,
+            "/path/to/file.csv",
+            multiLine=True,
+            rename_columns={"old_name": "new_name"}
+        )
+
+    Load a CSV file with a specific encoding:
+
+    >>> df = load_csv(spark, "/path/to/file.csv", encoding="ISO-8859-1")
+
+    Load a CSV file and keep only specific columns:
+
+    >>> df = load_csv(spark, "/path/to/file.csv", keep_columns=["col1", "col2"])
+
+    Load a CSV file and drop specific columns:
+
+    >>> df = load_csv(spark, "/path/to/file.csv", drop_columns=["col1", "col2"])
+
+    Load a CSV file with custom delimiter and multiline:
+
+    >>> df = load_csv(spark, "/path/to/file.csv", sep=";", multiLine=True)
+    """
+    try:
+        df = spark.read.csv(filepath, header=True, **kwargs)
+        logger.info(f"Loaded CSV file {filepath} with parameters {kwargs}")
+    except Exception as e:
+        error_message = f"Error loading file {filepath}: {e}"
+        logger.error(error_message)
+        raise Exception(error_message) from e
+
+    columns = [str(col) for col in df.columns]
+
+    # When multi_line is used it adds \r at the end of the final column
+    if kwargs.get("multiLine", False):
+        columns[-1] = columns[-1].replace("\r", "")
+        df = df.withColumnRenamed(df.columns[-1], columns[-1])
+
+    # Apply column transformations: keep, drop, rename
+    if keep_columns:
+        missing_columns = [col for col in keep_columns if col not in columns]
+        if missing_columns:
+            error_message = (
+                f"Columns {missing_columns} not found in DataFrame and cannot be kept"
+            )
+            logger.error(error_message)
+            raise ValueError(error_message)
+        df = df.select(*keep_columns)
+
+    if drop_columns:
+        for col in drop_columns:
+            if col in columns:
+                df = df.drop(col)
+            else:
+                error_message = (
+                    f"Column '{col}' not found in DataFrame and cannot be dropped"
+                )
+                logger.error(error_message)
+                raise ValueError(error_message)
+
+    if rename_columns:
+        for old_name, new_name in rename_columns.items():
+            if old_name in columns:
+                df = df.withColumnRenamed(old_name, new_name)
+            else:
+                error_message = (
+                    f"Column '{old_name}' not found in DataFrame and "
+                    f"cannot be renamed to '{new_name}'"
+                )
+                logger.error(error_message)
+                raise ValueError(error_message)
+
+    return df
+
+
+def truncate_external_hive_table(spark: SparkSession, table_identifier: str) -> None:
+    """Truncate an External Hive table stored on S3 or HDFS.
+
+    Parameters
+    ----------
+    spark
+        Active SparkSession.
+    table_identifier
+        The name of the Hive table to truncate. This can either be in the format
+        '<database>.<table>' or simply '<table>' if the current Spark session
+        has a database set.
+
+    Returns
+    -------
+    None
+        This function does not return any value. It performs an action of
+        truncating the table.
+
+    Raises
+    ------
+    ValueError
+        If the table name is incorrectly formatted, the database is not provided
+        when required, or if the table does not exist.
+    AnalysisException
+        If there is an issue with partition operations or SQL queries.
+    Exception
+        If there is a general failure during the truncation process.
+
+    Examples
+    --------
+    Truncate a Hive table named 'my_database.my_table':
+
+    >>> truncate_external_hive_table(spark, 'my_database.my_table')
+
+    Or, if the current Spark session already has a database set:
+
+    >>> spark.catalog.setCurrentDatabase('my_database')
+    >>> truncate_external_hive_table(spark, 'my_table')
+    """
+    try:
+        logger.info(f"Attempting to truncate the table '{table_identifier}'")
+
+        # Extract database and table name, even if only the table name is provided
+        db_name, table_name = extract_database_name(spark, table_identifier)
+
+        # Set the current database if a database was specified
+        if db_name:
+            spark.catalog.setCurrentDatabase(db_name)
+
+        # Check if the table exists before proceeding
+        if not spark.catalog.tableExists(table_name, db_name):
+            error_msg = f"Table '{db_name}.{table_name}' does not exist."
+            logger.error(error_msg)
+            raise ValueError(error_msg)
+
+        # Get the list of partitions
+        try:
+            partitions = spark.sql(f"SHOW PARTITIONS {db_name}.{table_name}").collect()
+        except Exception as e:
+            logger.warning(
+                f"Unable to retrieve partitions for '{db_name}.{table_name}': {e}",
+            )
+            partitions = []
+
+        if partitions:
+            logger.info(
+                f"Table '{table_identifier}' is partitioned. Dropping all partitions.",
+            )
+
+            # Drop each partition
+            for partition in partitions:
+                partition_spec = partition[
+                    0
+                ]  # e.g., partition is in format 'year=2023', etc.
+                spark.sql(
+                    f"ALTER TABLE {db_name}.{table_name} "
+                    f"DROP IF EXISTS PARTITION ({partition_spec})",
+                )
+
+        else:
+            logger.info(
+                f"Table '{table_identifier}' has no partitions or is not partitioned.",
+            )
+
+            # Overwrite with an empty DataFrame
+            original_df = spark.table(f"{db_name}.{table_name}")
+            schema: T.StructType = original_df.schema
+            empty_df = spark.createDataFrame([], schema)
+            empty_df.write.mode("overwrite").insertInto(f"{db_name}.{table_name}")
+
+        logger.info(f"Table '{table_identifier}' successfully truncated.")
+
+    except Exception as e:
+        logger.error(
+            f"An error occurred while truncating the table '{table_identifier}': {e}",
+        )
         raise
