@@ -1,23 +1,25 @@
 """Contains the logging configuration for files and method to initialise it."""
+
 import functools
-from functools import partial
 import logging
+from functools import partial
 from textwrap import dedent
 from typing import Callable, Dict, List, Optional
 
-from humanfriendly import format_timespan
 import pandas as pd
+from humanfriendly import format_timespan
 from pyspark import StorageLevel
 from pyspark.sql import DataFrame as SparkDF
 
-
 # A logger object named after module:
 # https://docs.python.org/3/howto/logging.html#advanced-logging-tutorial
 logger = logging.getLogger(__name__)
 
 
 LOG_DEV_LEVEL_NUM = 15
-logging.addLevelName(LOG_DEV_LEVEL_NUM, 'DEV')
+logging.addLevelName(LOG_DEV_LEVEL_NUM, "DEV")
+
+
 def log_dev(self, message, *args, **kwargs):  # noqa: E302
     """Create a custom log level between INFO and DEBUG named DEV.
 
@@ -26,6 +28,8 @@ def log_dev(self, message, *args, **kwargs):  # noqa: E302
     if self.isEnabledFor(LOG_DEV_LEVEL_NUM):
         # Yes, logger takes its '*args' as 'args'.
         self._log(LOG_DEV_LEVEL_NUM, message, args, **kwargs)  # noqa: W291
+
+
 logging.Logger.dev = log_dev  # noqa: E305
 
 
@@ -53,15 +57,17 @@ def init_logger_basic(log_level: int) -> None:
     """
     logging.basicConfig(
         level=log_level,
-        format='%(asctime)s %(levelname)s %(name)s: %(message)s',
-        datefmt='%Y-%m-%d %H:%M:%S',
+        format="%(asctime)s %(levelname)s %(name)s: %(message)s",
+        datefmt="%Y-%m-%d %H:%M:%S",
     )
 
-    logger.dev("""
+    logger.dev(
+        """
     Initialised logger for pipeline.
 
     Also have access to `logger.dev` by using this function.
-    """)
+    """,
+    )
 
 
 def init_logger_advanced(
@@ -121,9 +127,9 @@ def init_logger_advanced(
     """
     # Set default log format and date format if not provided
     if log_format is None:
-        log_format = '%(asctime)s %(levelname)s %(name)s: %(message)s'
+        log_format = "%(asctime)s %(levelname)s %(name)s: %(message)s"
     if date_format is None:
-        date_format = '%Y-%m-%d %H:%M:%S'
+        date_format = "%Y-%m-%d %H:%M:%S"
 
     # Prepare a formatter
     formatter = logging.Formatter(log_format, date_format)
@@ -140,8 +146,8 @@ def init_logger_advanced(
     for handler in handlers:
         if not isinstance(handler, logging.Handler):
             msg = (
-                f'Handler {handler} is not an instance of '
-                f'logging.Handler or its subclasses'
+                f"Handler {handler} is not an instance of "
+                f"logging.Handler or its subclasses"
             )
             raise ValueError(
                 msg,
@@ -158,7 +164,7 @@ def init_logger_advanced(
             datefmt=date_format,
         )
 
-    logger.debug('Initialised logger for pipeline.')
+    logger.debug("Initialised logger for pipeline.")
 
 
 def timer_args(
@@ -182,10 +188,10 @@ def timer_args(
         Dictionary of arguments to pass to specifc codetiming package Timer.
     """
     return {
-        'name': name,
-        'text': lambda secs: name + f': {format_timespan(secs)}',
-        'logger': logger,
-        'initial_text': 'Running {name}',
+        "name": name,
+        "text": lambda secs: name + f": {format_timespan(secs)}",
+        "logger": logger,
+        "initial_text": "Running {name}",
     }
 
 
@@ -322,6 +328,7 @@ def log_spark_df_schema(
     |-- expenditure: double (nullable = true)
     ```
     """  # noqa: E501
+
     def decorator_function(func):
         @functools.wraps(func)
         def wrapper_decorator(*args, **kwargs):
@@ -329,8 +336,10 @@ def log_spark_df_schema(
             func_name = func.__name__
 
             if log_schema_on_input:
-                if not kwargs.get('df'):
-                    logger.warning(dedent(f"""
+                if not kwargs.get("df"):
+                    logger.warning(
+                        dedent(
+                            f"""
                     Cannot find `df` in keyword named arguments.
 
                     To use the log_spark_df_schema decorator with the function
@@ -338,19 +347,29 @@ def log_spark_df_schema(
                     * it must have a parameter called df that is a spark dataframe.
                     * it must be called specifying the argument names e.g.
                     {func_name}(df=input_df, ... )
-                    """))  # noqa: E501
-                elif isinstance(kwargs['df'], SparkDF):
-                    schema = kwargs['df']._jdf.schema().treeString()
-                    logger.info(f'Schema of dataframe before {func_name}:\n{schema}')  # noqa: E501
+                    """,
+                        ),
+                    )  # noqa: E501
+                elif isinstance(kwargs["df"], SparkDF):
+                    schema = kwargs["df"]._jdf.schema().treeString()
+                    logger.info(
+                        f"Schema of dataframe before {func_name}:\n{schema}",
+                    )  # noqa: E501
                 else:
-                    logger.warning(dedent(f"""
+                    logger.warning(
+                        dedent(
+                            f"""
                     {func_name} keyword argument `df` has type {type(kwargs['df'])}.
 
                     Cannot print spark schema for this type of object.
-                    """))  # noqa: E501
+                    """,
+                        ),
+                    )  # noqa: E501
 
             else:
-                logger.info(f'Not printing schema of dataframe before {func_name}')  # noqa: E501
+                logger.info(
+                    f"Not printing schema of dataframe before {func_name}",
+                )  # noqa: E501
 
             # Run the decorated function in its normal way, but catch its
             # output so its schema can be printed.
@@ -361,9 +380,14 @@ def log_spark_df_schema(
             # fail.
             if isinstance(df_return, SparkDF):
                 schema = df_return._jdf.schema().treeString()
-                logger.info(f'Schema of dataframe after {func_name}:\n{schema}')  # noqa: E501
+                logger.info(
+                    f"Schema of dataframe after {func_name}:\n{schema}",
+                )  # noqa: E501
             else:
-                logger.warning(f'{func_name} should return a spark dataframe for decorator, but returned {type(df_return)}')  # noqa: E501
+                logger.warning(
+                    f"{func_name} should return a spark dataframe for decorator, "
+                    f"but returned {type(df_return)}",
+                )
 
             return df_return
 
@@ -409,19 +433,23 @@ def log_rows_in_spark_df(func: Callable) -> Callable:
     as persisting the output dataframe prior to counting. The input dataframe
     is also unpersisted from memory prior to the decorator completing.
     """
-    logger.debug("""
+    logger.debug(
+        """
     log_rows_in_spark_df caches and persists spark dataframes to memory.
     It also performs count operations. Both of these could have an adverse
     effect on pipelines if used incorrectly, so use as decorator with care.
-    """)
+    """,
+    )
 
     @functools.wraps(func)
     def wrapper_decorator(*args, **kwargs):
         # Define the name of the function being decorated for use in logs.
         func_name = func.__name__
 
-        if not kwargs.get('df'):
-            logger.warning(dedent(f"""
+        if not kwargs.get("df"):
+            logger.warning(
+                dedent(
+                    f"""
             Cannot find `df` in keyword named arguments.
 
             To use the log_rows_in_spark_df decorator with the function
@@ -429,21 +457,29 @@ def log_rows_in_spark_df(func: Callable) -> Callable:
             * it must have a parameter called df that is a spark dataframe.
             * it must be called specifying the argument names e.g.
             {func_name}(df=input_df, ... )
-            """))
-        elif isinstance(kwargs['df'], SparkDF):
+            """,
+                ),
+            )
+        elif isinstance(kwargs["df"], SparkDF):
             # If not already cached, cache the dataframe prior to counting to
             # allow more efficient processing in function. Is unpersisted at
             # end of decorator.
-            if not kwargs['df'].is_cached:
-                kwargs['df'].cache()
+            if not kwargs["df"].is_cached:
+                kwargs["df"].cache()
 
-            logger.info(f"Rows in dataframe before {func_name} : {kwargs['df'].count()}")  # noqa: E501
+            logger.info(
+                f"Rows in dataframe before {func_name} : {kwargs['df'].count()}",
+            )  # noqa: E501
         else:
-            logger.warning(dedent(f"""
+            logger.warning(
+                dedent(
+                    f"""
             {func_name} keyword argument `df` has type {type(kwargs['df'])}.
 
             Cannot count rows for this type of object.
-            """))
+            """,
+                ),
+            )
 
         # Run the decorated function in its normal way, but catch its output
         # so it can be counted.
@@ -458,14 +494,19 @@ def log_rows_in_spark_df(func: Callable) -> Callable:
             # decorator. Therefore, we don't want it to get pushed onto disk
             # (and incur an expensive swap operation).
             df_return.persist(StorageLevel.MEMORY_ONLY)
-            logger.info(f'Rows in dataframe after {func_name}  : {df_return.count()}')  # noqa: E501
+            logger.info(
+                f"Rows in dataframe after {func_name}  : {df_return.count()}",
+            )  # noqa: E501
 
         else:
-            logger.warning(f'{func_name} should return a spark dataframe for decorator, but returned {type(df_return)}')  # noqa: E501
+            logger.warning(
+                f"{func_name} should return a spark dataframe for decorator, "
+                f"but returned {type(df_return)}",
+            )
 
-        if kwargs.get('df'):
+        if kwargs.get("df"):
             # Unpersist the cached input df to manage memory.
-            kwargs['df'].unpersist()
+            kwargs["df"].unpersist()
 
         return df_return
 
@@ -505,13 +546,14 @@ def add_warning_message_to_function(
     Warning my_func: here be dragons...
     ```
     """  # noqa: E501
+
     def decorator_function(func):
         @functools.wraps(func)
         def wrapper_decorator(*args, **kwargs):
             # Define the name of the function being decorated for use in logs.
             func_name = func.__name__
             if message:
-                logger.warning(f'{func_name}: {message}')
+                logger.warning(f"{func_name}: {message}")
 
             # Run the decorated function in its normal way.
             output = func(*args, **kwargs)
@@ -528,5 +570,5 @@ def add_warning_message_to_function(
 
 not_undergone_functional_test_warning = partial(
     add_warning_message_to_function,
-    message='is unit tested, but not formally end-to-end tested.',
+    message="is unit tested, but not formally end-to-end tested.",
 )
