@@ -5,7 +5,7 @@ import pytest
 from rdsa_utils.validation import *
 
 
-@pytest.mark.skip(reason='test shell')
+@pytest.mark.skip(reason="test shell")
 class TestApplyValidation:
     """Tests for the apply_validation function."""
 
@@ -14,7 +14,7 @@ class TestApplyValidation:
         pass
 
 
-@pytest.mark.skip(reason='decorator wrapper of list_convert function.')
+@pytest.mark.skip(reason="decorator wrapper of list_convert function.")
 class TestListConvertValidator:
     """Tests for the list_convert_validator function."""
 
@@ -32,7 +32,7 @@ class TestAllowedDateFormat:
 
     def test_expected(self):
         """Test expected functionality."""
-        date = 'Feb 2022'
+        date = "Feb 2022"
 
         actual = allowed_date_format(date)
 
